@@ -67,6 +67,7 @@ typedef struct r_flag_t {
 
 typedef bool (*RFlagExistAt)(RFlag *f, const char *flag_prefix, ut16 fp_size, ut64 off);
 typedef RFlagItem* (*RFlagGet)(RFlag *f, const char *name);
+typedef RFlagItem *(*RFlagGetAtAddr) (RFlag *f, ut64);
 typedef RFlagItem* (*RFlagGetAt)(RFlag *f, ut64 addr, bool closest);
 typedef RFlagItem* (*RFlagSet)(RFlag *f, const char *name, ut64 addr, ut32 size);
 typedef bool (*RFlagUnset)(RFlag *f, RFlagItem *item);
@@ -103,7 +104,7 @@ R_API void r_flag_list(RFlag *f, int rad, const char *pfx);
 R_API bool r_flag_exist_at(RFlag *f, const char *flag_prefix, ut16 fp_size, ut64 off);
 R_API RFlagItem *r_flag_get(RFlag *f, const char *name);
 R_API RFlagItem *r_flag_get_i(RFlag *f, ut64 off);
-R_API RFlagItem *r_flag_get_i2(RFlag *f, ut64 off);
+R_API RFlagItem *r_flag_get_by_spaces(RFlag *f, ut64 off, ...);
 R_API RFlagItem *r_flag_get_at(RFlag *f, ut64 off, bool closest);
 R_API RList *r_flag_all_list(RFlag *f, bool by_space);
 R_API const RList* /*<RFlagItem*>*/ r_flag_get_list(RFlag *f, ut64 off);
@@ -175,7 +176,7 @@ static inline bool r_flag_space_is_empty(RFlag *f) {
 #define r_flag_space_foreach(f, it, s) r_spaces_foreach (&(f)->spaces, (it), (s))
 
 /* tags */
-R_API RList *r_flag_tags_list(RFlag *f);
+R_API RList *r_flag_tags_list(RFlag *f, const char *name);
 R_API RList *r_flag_tags_set(RFlag *f, const char *name, const char *words);
 R_API void r_flag_tags_reset(RFlag *f, const char *name);
 R_API RList *r_flag_tags_get(RFlag *f, const char *name);
@@ -188,6 +189,7 @@ R_API bool r_flag_zone_del(RFlag *fz, const char *name);
 R_API bool r_flag_zone_around(RFlag *fz, ut64 addr, const char **prev, const char **next);
 R_API bool r_flag_zone_list(RFlag *fz, int mode);
 R_API bool r_flag_zone_reset(RFlag *f);
+R_API RList *r_flag_zone_barlist(RFlag *f, ut64 from, ut64 bsize, int rows);
 
 #endif
 
